/*
 * Licensed to the OpenAirInterface (OAI) Software Alliance under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The OpenAirInterface Software Alliance licenses this file to You under
 * the OAI Public License, Version 1.1  (the "License"); you may not use this file
 * except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.openairinterface.org/?page_id=698
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *-------------------------------------------------------------------------------
 * For more information about the OpenAirInterface (OAI) Software Alliance:
 *      contact@openairinterface.org
 */

#include "../../../../src/xApp/e42_xapp_api.h"
#include "../../../../src/util/alg_ds/alg/defer.h"
#include "../../../../src/util/time_now_us.h"

#include "../../../../src/sm/pkt_sm/pkt_sm_id.h"

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

/*static
void sm_cb_pkt(sm_ag_if_rd_t* rd)
{
  assert(rd != NULL);
  assert(rd->type == PKT_STATS_V0);

  int64_t now = time_now_us();
  printf("PKT ind_msg latency = %ld \n", now - rd->pkt_stats.msg.tstamp);
}*/

static
sm_ag_if_wr_ctrl_t fill_pkt_sm_ctrl_req(uint16_t ran_func_id)
{
  assert(ran_func_id == 149);

  sm_ag_if_wr_ctrl_t wr = {0};
  wr.type = PKT_CTRL_REQ_V0;
  return wr;
}

int main(int argc, char *argv[])
{

  fr_args_t args = init_fr_args(argc, argv);
  
  // rtt parameters
  int64_t send_time;
  int64_t ack_time;
  int64_t rtt_mean = 0;
  int64_t diff_time;
  int64_t nr_of_pkts = 100;
  
  //Init the xApp
  init_xapp_api(&args);
  sleep(1);

  e2_node_arr_t nodes = e2_nodes_xapp_api();
  defer({ free_e2_node_arr(&nodes); });

  assert(nodes.len > 0);

  printf("Connected E2 nodes = %d\n", nodes.len);

  // PKT indication
  inter_xapp_e i_0 = ms_5;
  sm_ans_xapp_t* pkt_handle = NULL;
  sm_ans_xapp_t pkt_rtt;
  if(nodes.len > 0){
    pkt_handle = calloc( nodes.len, sizeof(sm_ans_xapp_t) ); 
    assert(pkt_handle  != NULL);
  }

  for (int i = 0; i < nodes.len; i++) {
    e2_node_connected_t* n = &nodes.n[i];
    for (size_t j = 0; j < n->len_rf; j++)
      printf("Registered node %d ran func id = %d \n ", i, n->ack_rf[j].id);

    //pkt_handle[i] = report_sm_xapp_api(&nodes.n[i].id, n->ack_rf[7].id, i_0, sm_cb_pkt);
    //assert(pkt_handle[i].success == true);
    pkt_ctrl_req_data_t pkt_ctrl = {0};
    pkt_ctrl.msg.action = 24;
    strcpy(pkt_ctrl.msg.pkt,"Oi"); 
    for (int pkt = 0; pkt < nr_of_pkts; pkt++) {
      send_time = time_now_us();
      pkt_rtt = control_sm_xapp_api(&nodes.n[i].id, SM_PKT_ID, &pkt_ctrl);
      while(!pkt_rtt.success){
        printf("Resultado do bool: %d\n", pkt_rtt.success);
      }
      ack_time = time_now_us();
      diff_time = (ack_time-send_time); 
      rtt_mean += diff_time;
      printf("\n\nRTT Control Message (%d) = %d us\n\n",pkt, diff_time);
    }
    
    //free_pkt_ctrl_msg(&ctrl_msg_add.pkt_ctrl.msg);
    rtt_mean = rtt_mean/nr_of_pkts;
    printf("\n\n RTT mean:%d us\n\n", rtt_mean);
  }

  sleep(10);

  /*for(int i = 0; i < nodes.len; ++i){
    // Remove the handle previously returned
    rm_report_sm_xapp_api(pkt_handle[i].u.handle);
  }*/

  if(nodes.len > 0){
    free(pkt_handle);
  }

  //Stop the xApp
  while(try_stop_xapp_api() == false)
    usleep(1000);

  printf("Test xApp run SUCCESSFULLY\n");
}